% Seven point Gaussian quadrature of 1/|r-r0| over a triangle.

% a,b,c - coordinates of 3 vertices, each is a vector 1X3
% area - area of the triangle
% r0 - test point coordinates, vector 1X3

% Triangle barycentric coordinates - alpha, beta, gamma=1-alpha-beta
% r=alpha*a+beta*b+gamma*c
% dA = 2A*dalpha*dbeta

function I=triangle_Gaussian_quadrature(a,b,c,area,r0)

% Four point Quadrature Rule, the columns are alpha, beta, gamma, w (weight)
% QR4=[1/3 1/3 1/3 -0.5625; 
%     0.6 0.2 0.2 0.52083333;
%     0.2 0.6 0.2 0.52083333;
%     0.2 0.2 0.6 0.52083333];

% Seven point Quadrature Rule, the columns are alpha, beta, gamma, w (weight)
QR7=[0.33333333 0.33333333 0.33333333 0.225;
    0.05971587 0.47014206 0.47014206 0.13239415;
    0.47014206 0.05971587 0.47014206 0.13239415;
    0.47014206 0.47014206 0.05971587 0.13239415;
    0.79742698 0.10128650 0.10128650 0.12593918;
    0.10128650 0.79742698 0.10128650 0.12593918;
    0.10128650 0.10128650 0.79742698 0.12593918];


% The seven quadrature points inside the triangles
pq=zeros(7,3);
pq(:,1)=QR7(:,1:3)*[a(1); b(1); c(1)]; % x coordinates
pq(:,2)=QR7(:,1:3)*[a(2); b(2); c(2)]; % y coordinates
pq(:,3)=QR7(:,1:3)*[a(3); b(3); c(3)]; % z coordinates

% Integrated function evaluated at each quadrature point
f=zeros(7,1);
f=1./sqrt(sum([r0(1)-pq(:,1) r0(2)-pq(:,2) r0(3)-pq(:,3)].^2,2));
% f=(x0-px)+(y0-py); % Integral of this function is zero

% Gaussian Quadrature integral
I=area*f'*QR7(:,4);
end

