function loop(p,H0,Hm,w)

figHandle = figure('name','Wave animation','units','normalized','outerposition',[0 0.05 1 0.9]);
set(figHandle,'units','pixels');
pos = get(figHandle, 'Position');
width = pos(3);
height = pos(4);

[X,Y] = meshgrid(-p.meshSize/2:p.meshSize/2-1,-p.meshSize/2:p.meshSize/2-1); %create mesh

floor = surf(X,Y,-p.h,'FaceColor','interp','edgecolor','none','edgelighting','none'); % plot seafloor
hold on
sea = surf(X,Y,randn(p.meshSize),'EdgeColor','interp','edgecolor','none','edgelighting','none');

bluemap = blue_colormap; % blue colormap for waves
seamap = zeros(p.meshSize, p.meshSize, 3);
sandmap = figdesign(gca,gcf,p); % colormap for seafloor and general figure design
floor.CData = sandmap;
alpha(sea,.75)
time_disp = title('Time','FontSize',11,'fontweight','bold');

S.pb = uicontrol('string','Skip 10 seconds','callback',{@pb_skip},'units','pixels',...
                 'fontsize',11,'fontweight','bold','position',[width/2+130, height-50, 140, 30]); %skip button
S.pb = uicontrol('string','Stop Animation!','callback',{@pb_del_fig},'units','pixels',...
                 'fontsize',11,'fontweight','bold','position',[width/2+280, height-50, 140, 30]); %stop button

if p.saveVid
    v = VideoWriter(p.fileName);
    v.FrameRate = p.frameRate;
    open(v)
end

skip = -2; % skip first two non-physical iteration
prev_shock = zeros(p.meshSize);
after_shock = 0;

time_shift = 0;
tic

while(1)
    time = toc + time_shift;
    
    if skip
        skip = skip +1;
    end
    
    if ~ishandle(sea)  % check if the figure exists (for stopping scenario)
        break
    end
    
    Hkt = H0.*exp(1i.*w*time) + Hm.*exp(-1i.*w*time); % update surface by w
    Ht = real(ifft2(Hkt)); % Generate HeightField at time t using ifft
    for m = 1:p.meshSize
        for n = 1:p.meshSize
             signCorrection = mod(m+n-2, 2); % fft shift
             if signCorrection
                 Ht(m,n) = -Ht(m,n);
             end
        end
    end
    
    if ~after_shock
        [shock_map, after_shock] = shock_boundary(p, Ht, prev_shock);
        Ht = Ht .* shock_map;
        prev_shock = shock_map;
    end
    
    HtN = Ht - min(Ht(:));
    HtN = HtN / max(HtN(:)) - .5; % Ht normilzed over +-.5
    HtN = HtN.^2 + .5; % strength center (blue) and shift to [0 1]
    
    for m = 1:p.meshSize
        for n = 1:p.meshSize
            seamap(m,n,1) = bluemap(round(HtN(m,n)*255+1),1);
            seamap(m,n,2) = bluemap(round(HtN(m,n)*255+1),2);
            seamap(m,n,3) = bluemap(round(HtN(m,n)*255+1),3);
        end
    end
    
    if skip
        sea.ZData = zeros(p.meshSize);
    else
        sea.ZData = Ht;
    end
    sea.CData = seamap;
    
    if p.saveVid
        frame = getframe(figHandle,[pos(1)+1 pos(1)+1 pos(3)-2 pos(4)-2]);
        writeVideo(v,frame)
    end
    
    pause(1/p.frameRate)
    set(time_disp,'String',['Time: ',num2str(time,'%.1f'),' sec'])
end


function [] = pb_skip(varargin)  % Callback for pushbutton
    time_shift = time_shift + 10;  % skip 10 seconds
end

function [] = pb_del_fig(varargin)  % Callback for pushbutton
    delete(sea)  % Delete the figure
end

close

if p.saveVid
    close(v)
end

end