%% Simulating Ocean Waves using finite elements method
close all
clear
clc

p = set_parameters; % p is struct containing all parameters
shore = shore_selection; % shore shape selection
CPU = processor_selection; % GPU/CPU selection

if CPU
    p.h = -generate_seafloor(p,shore); % depth ( = - seafloor elevation)
    [S,k] = generate_wave_spectrum(p); % S is real domain data k is in the dual one
    [H0,Hm] = initial_surface(p,S); % in k space. Hm is mirored value of H0
    w = (p.g.*k*tanh(p.h*k))^.5; % dispertion relation
    loop(p,H0,Hm,w)
else % GPU
    reset(gpuDevice(1)) % clear GPU's memory
    p = set_gpu_parameters(p);
    p.h = -generate_seafloor(p,shore); % depth ( = - seafloor elevation)
    gh = gpuArray(p.h); % copy arrays to GPU
    [S,k] = generate_wave_spectrum(p);
    gk = gpuArray(k);
    [H0,Hm] = initial_surface(p,S); % in k space. Hm is mirored value of H0
    gH0 = gpuArray(H0);
    gHm = gpuArray(Hm);
    gw = (p.g.*gk*tanh(gh*gk))^.5; % dispertion relation
    loop_gpu(p,gH0,gHm,gw)
end