%
%
% The program was written by Dan V. Regelman as a project in the
% Computational Physics Course at the Technion given by Dr. Joan Adler,
% Physics Department, 2000
%


function out_data=sdemo(action,s)

% Possible actions:

% INITIALIZE
% CHANGEDEMO
% INFO
% GETPOTENTIAL
% SOLVE
% DRAG
% DRAGVERT
% DRAGHORIZ
% DRAW
% DRAW_FINISH

if nargin<1,
	action='initialize';
end;

if strcmp(action,'initialize'),

        shh=get(0,'ShowHiddenHandles');
        set(0,'ShowHiddenHandles','on');
	figNumber=figure(...
		'name','Shrodinger equation solution demonstration',...
		'handleVisibility','callback',...
		'integerHandle','off',...
		'numberTitle','off',...
		'Tag','Srodemo');

	%===========================
	% Set up the axes 
	axes(...
		'units','normalized',...
		'position',[0.1 0.1 0.6 0.8],...
		'Xtick',[],'Ytick',[],...
		'box','on');
	set(figNumber,'defaultAxesPosition',[0.1 0.1 0.6 0.8]);
	Ahandle=subplot(1,1,1);
	set(gca,...
		'units','normalized',...
		'position',[0.1 0.1 0.6 0.8],...
		'XTick',[],'YTick',[],...
		'Box','on');

	%===================================
	% Informantion for all buttons

	labelColor=[0.8 0.8 0.8];
	yInitPos=0.9;
	menutop=0.95;
	btnTop=0.6;
	top=0.75;
	left=0.785;
	btnWid=0.175;
	btnHt=0.06;
	textHeight=0.05;
	textWidth=0.075;
	spacing=0.019;

    %====================================
    % The CONSOLE frame
    frmBorder=0.019; frmBottom=0.04; 
    frmHeight = 0.92; frmWidth = btnWid;
    yPos=frmBottom-frmBorder;
    frmPos=[left-frmBorder yPos frmWidth+2*frmBorder frmHeight+2*frmBorder];
    h=uicontrol( ...
        'Style','frame', ...
        'Units','normalized', ...
        'Position',frmPos, ...
        'BackgroundColor',[0.5 0.5 0.5]);

    %====================================
    % The demonstration selection menu
    btnNumber=1;
    yPos=menutop-(btnNumber-1)*(btnHt+spacing);
    btnPos=[left yPos-btnHt btnWid btnHt];
    labelStr='Square well|Double well|Parabolic|Draw your own';
    callbackStr='sdemo(''changedemo'');';
    demoHndl=uicontrol( ...
        'Style','popupmenu', ...
        'Units','normalized', ...
        'Position',btnPos, ...
        'String',labelStr, ...
        'Interruptible','on', ...
        'Callback',callbackStr);
        
    %===================================
    % Put labeles 
    LabelStrings=['mass   ';...
                  'depth  ';...
		  'length ';...
		  'N      ';...
                  'states ';...
		  'nds_X  ';...
                  'nds_Y  ';];
        
    for btnNumber=1:5,
        yPos=menutop-(btnNumber-1)*(btnHt+spacing);
    	top = yPos - btnHt - spacing;
    	labelWidth = frmWidth-textWidth-.01;
    	labelBottom=top-textHeight;
    	labelLeft = left;
    	labelPos = [labelLeft labelBottom labelWidth textHeight];
    	h = uicontrol( ...
       	 	'Style','text', ...
       		 'Units','normalized', ...
       		 'Position',labelPos, ...
       		 'Horiz','left', ...
       		 'String',LabelStrings(btnNumber,:), ...
       		 'Interruptible','off', ...
       		 'BackgroundColor',[0.5 0.5 0.5], ...
       		 'ForegroundColor','white');
	end


	TextStrings={0.067,-0.4,500.0,100,3,[0 150 350 500],[0 -0.4 0]};
        CallbackStr= 'sdemo(''solve'',1)';
        

	for btnNumber=1:7,
	       yPos=menutop-(btnNumber-1)*(btnHt+spacing);
               top = yPos - btnHt - spacing;
               tWidth = frmWidth-textWidth-.01;
    	       tBottom=top-textHeight;
    	       tLeft = left+labelWidth-0.01;
    	       tPos = [tLeft tBottom tWidth textHeight];
    	       tPhys(btnNumber) = uicontrol( ...
			      'Style','edit', ...
			      'Units','normalized', ...
			      'Position',tPos, ...
			      'Horiz','right', ...
			      'String',num2str(TextStrings{btnNumber}), ...
			      'Interruptible','on', ...
			      'BackgroundColor','white', ...
			      'ForegroundColor','black',...
			      'Callback',CallbackStr);
         end
	 set(tPhys(6),'visible','off');
         set(tPhys(7),'visible','off');
   
    %====================================
    % The boundary conditions list caption
    tmp=uicontrol( ...
        'Style','text', ...
        'Units','normalized', ...
        'Position',[left frmBottom+3*btnHt+3*spacing btnWid btnHt/1.5], ...
        'String','Boundaries');

    %====================================
    % The boundary conditions list
    callbackStr='sdemo(''solve'',1)';
    boundHndl=uicontrol( ...
        'Style','popupmenu', ...
        'Units','normalized', ...
        'Position',[left frmBottom+2*btnHt+2*spacing btnWid btnHt], ...
        'String','Fixed | Periodic', ...
        'Callback',callbackStr);
      

    %====================================
    % The INFO button
    labelStr='Info';
    callbackStr='web info.html';
    helpHndl=uicontrol( ...
        'Style','pushbutton', ...
        'Units','normalized', ...
        'Position',[left frmBottom+btnHt+spacing btnWid btnHt], ...
        'String',labelStr, ...
        'Callback',callbackStr);

    %====================================
    % The CLOSE button
    labelStr='Close';
    callbackStr='close(gcf)';
    closeHndl=uicontrol( ...
        'Style','pushbutton', ...
        'Units','normalized', ...
        'Position',[left frmBottom btnWid btnHt], ...
        'String',labelStr, ...
        'Callback',callbackStr);

    hndList=[tPhys demoHndl boundHndl];
    set(figNumber,...
        'Visible','on',...
        'UserData',hndList);
   
    %====================================
    POT=sdemo('getpotential');
    sdemo('solve',1);
    out_data=0;
    return


elseif strcmp(action,'getpotential'),
   hndList=get(gcf,'UserData');
   tPhys=hndList(1:7);
   N=str2num(get(tPhys(4),'string'));     
   xnodes=str2num(get(tPhys(6),'string'));
   ynodes=str2num(get(tPhys(7),'string'));  
   POT=zeros(N,1);
   dnode=N/(xnodes(end)-xnodes(1));
   xnodes=xnodes-xnodes(1);
   for i=1:length(xnodes)-1,
     POT(fix(xnodes(i)*dnode)+1:fix(xnodes(i+1)*dnode))=ynodes(i);
   end
   out_data=POT;
   return

elseif strcmp(action,'solve'),
   hndList=get(gcf,'UserData');
   tPhys=hndList(1:7);
   boundHndl=hndList(9);

   if s==1,
       %Scale the nodes according to new length and potential settings
       old_xnodes=str2num(get(tPhys(6),'string'));
       Lmax=str2num(get(tPhys(3),'string'));
       new_xnodes= num2str(old_xnodes/max(old_xnodes)*Lmax);
       set(tPhys(6),'string',new_xnodes);
       old_ynodes=str2num(get(tPhys(7),'string'));
       Vmax=-str2num(get(tPhys(2),'string'));
       Vlow=min(old_ynodes);
       Vhigh=max(old_ynodes);
       num=max(abs(Vlow),abs(Vhigh))*sign(Vmax);
       new_ynodes=num2str(old_ynodes/num*Vmax);
       set(tPhys(7),'string',new_ynodes);
   end

   N=str2num(get(tPhys(4),'string'));
   m=str2num(get(tPhys(1),'string'));
   states=str2num(get(tPhys(5),'string'));
   xnodes=str2num(get(tPhys(6),'string'));
   ynodes=str2num(get(tPhys(7),'string')); 
   dx=(xnodes(end)-xnodes(1))/(N-1);
   POT=sdemo('getpotential');
   
   % Computation kernel
   h22m=3.78/m;
   H=sparse(N);
   dx2=h22m/dx^2;
   H=spdiags([-ones(N,1)*dx2 2*ones(N,1)*dx2+POT -ones(N,1)*dx2],-1:1,N,N);
 
%****** Add periodic boundary conditions if requested
   if get(boundHndl,'Value')==2,
       H(1,N)=-dx2;
       H(N,1)=-dx2;
    end

  %[v,e]=eigs(H,'BE',struct('tol',1e-5));
   [v,e]=eig(full(H));
   e=diag(e);   
   [e,index]=sort(e);
   v=v(:,index);

   scale=max(abs(ynodes))/2; 
   x_vec=xnodes(1):dx:xnodes(end);

   cla;
   linetype=['r--';' r-'];
   for i=1:states,
         if i==2, hold on;end
         maxv=find(abs(v(:,i))==max(abs(v(:,i))));
         ps=sign(v(maxv,i))';
         plot(x_vec,v(:,i)*ps*scale+e(i),linetype(1+mod(i,2),:));
         h=line([min(x_vec) max(x_vec)],[e(i) e(i)]);
         set(h,'LineWidth',2,'Color','green');
    end

   clear HndlV;   % Remove old handles for Potential
   W_x=reshape(ones(2,1)*xnodes,[1 2*length(xnodes)]);
   W_x=W_x(2:(end-1));
   W_y=reshape(ones(2,1)*ynodes,[1 2*length(ynodes)]);
   W_y=W_y(1:end);
   set(gcf,'handlevisibility','callback'); 

   for i=1:length(W_x)-1,
       HndlV(i)=line([W_x(i) W_x(i+1)],[W_y(i) W_y(i+1)]);
       set(HndlV(i),'LineWidth',2,'Color','blue');
       set(HndlV(i),'buttondownfcn','sdemo(''drag'');');
   end   
   hold off
   set(gcf,'UserData',[get(gcf,'UserData') HndlV NaN length(W_x)-1]);
   return

elseif strcmp(action,'changedemo'),
     Hndls=get(gcf,'Userdata');
     tPhys=Hndls(1:7);
     demoHndl=Hndls(8);
     switch get(demoHndl,'Value')
           case 1,
                 xnodes=[0 150 350 500]; 
                 ynodes=[0 -0.4 0];
                 set(tPhys(6),'String',num2str(xnodes));
                 set(tPhys(7),'String',num2str(ynodes));       
                 sdemo('solve',1);
            case 2,
                  xnodes=[0 155 255 275 375 500];
	         ynodes=[0 -0.4 -0.03 -0.4 0];
                 set(tPhys(6),'String',num2str(xnodes));
                 set(tPhys(7),'String',num2str(ynodes));
	         sdemo('solve',1);
	     case 3,
                  xnodes=[0:500];
		  ynodes=(xnodes/250-1).^2*0.4;
                  set(tPhys(6),'String',num2str(xnodes));
                  set(tPhys(7),'String',num2str(ynodes));
                  sdemo('solve',1);
	     case 4,
                 sdemo('Draw',2);
                
     end
     
         return

elseif strcmp(action,'drag')
     the_line=gco;
     tmp=get(gcf,'UserData');
     tmp(end-1)=gco;
     set(gcf,'UserData',tmp); 
 % define if drag vertical or horizontal
            pt=get(gca,'currentpoint');
            ptx=pt(1,1);pty=pt(1,2);
            set(gca,'units','pixels');
            pos=get(gca,'position');
            set(gca,'units','normalized');
            xl=get(gca,'xlim');
            W=get(the_line,'Xdata');
            set(gca,'units','pixels');
            W_p=get(the_line,'Xdata');
            set(the_line,'Color','red');
 
            if abs(W(1)-W(2))<10,
 
                   ClbStr1='sdemo(''dragHoriz'',2);';
                   ClbStr2='sdemo(''dragHoriz'',3);';
            else
		    ClbStr1='sdemo(''dragVert'',2);';
		    ClbStr2='sdemo(''dragVert'',3);';
            end 
            set(gcf,'windowbuttonmotionfcn',ClbStr1);
            set(gcf,'windowbuttonupfcn',ClbStr2);
            return
         
elseif strcmp(action,'dragVert')
        tmp=get(gcf,'UserData');
	the_line=tmp(end-1);
        N=tmp(end);
        tPhys=tmp(1:7);

        all_lines=tmp(end-N-1:end-2);        
        the_line_index=find(the_line==all_lines);
        status=s;
	if status==2,
	      pt=get(gca,'currentpoint');
	      new_y=pt(1,2);
	      set(the_line,'erasemode','xor');
              set(the_line,'Ydata',[new_y new_y]);
	      if the_line_index>1,
                     old_y=get(all_lines(the_line_index-1),'Ydata');
                     set(all_lines(the_line_index-1),'Ydata',[old_y(1) new_y]);
	      end
              if the_line_index<N,
                     old_y=get(all_lines(the_line_index+1),'Ydata');
                     set(all_lines(the_line_index+1),'Ydata',[new_y old_y(2)])
              end
	elseif status==3,
            st='';
            for i=1:N,
               tmp=get(all_lines(i),'Ydata');
               tmp=tmp(1,2);
                if (fix(i/2)-i/2)~=0,
                    st=[st num2str(tmp) ' '];
                end
            end
            set(tPhys(7),'String',st);
            sdemo('solve',0);
            set(gcf,'windowbuttonmotionfcn','');
            set(gcf,'windowbuttonupfcn',''); 
        end



elseif strcmp(action,'dragHoriz')
        tmp=get(gcf,'UserData');
	the_line=tmp(end-1);
        N=tmp(end);
        tPhys=tmp(1:7);

        all_lines=tmp(end-N-1:end-2);        
        the_line_index=find(the_line==all_lines);
        status=s;
	if status==2,
	      pt=get(gca,'currentpoint');
	      new_x=pt(1,1);
	      set(the_line,'erasemode','xor');
            %**** Check validity
               x_left=get(all_lines(the_line_index-1),'Xdata');
               x_right=get(all_lines(the_line_index+1),'Xdata');
               if new_x<x_left(1)+10,
                      new_x=x_left(1)+10;
               elseif new_x>x_right(2)-10,
		      new_x=x_right(2)-10;
               end

              set(the_line,'Xdata',[new_x new_x]);
	      if the_line_index>1,
                     old_x=get(all_lines(the_line_index-1),'Xdata');
                     set(all_lines(the_line_index-1),'Xdata',[old_x(1) new_x]);
	      end
              if the_line_index<N,
                     old_x=get(all_lines(the_line_index+1),'Xdata');
                     set(all_lines(the_line_index+1),'Xdata',[new_x old_x(2)])
              end
	elseif status==3,
            st='';
            for i=1:N,
               tmp=get(all_lines(i),'Xdata');
               tmp=tmp(1,1);
                if (fix(i/2)-i/2)~=0,
                    st=[st num2str(tmp) ' '];
                end          
            end
            
            tmp=get(all_lines(N),'Xdata');
            tmp=tmp(1,2);
              st=[st num2str(tmp)];

            set(tPhys(6),'String',st);
            sdemo('solve',0);
            set(gcf,'windowbuttonmotionfcn','');
            set(gcf,'windowbuttonupfcn',''); 
        end


elseif strcmp(action,'Draw')
        
        helpstr=['Left button to locate a point,'
                 'Right button to end.          '];
        tmp=get(0,'Currentfigure');
        pos=get(tmp,'position');
        hndl=helpdlg(helpstr,'Drawing mode');
        pos2=get(hndl,'position');
        set(hndl,'position',[pos(1) pos(2)-pos(3)/2.5 pos2(3) pos2(4)]); 
        set(0,'Currentfigure',tmp);
        tmp=get(gcf,'UserData');
	tPhys=tmp(1:7);
        cla;axis([0 1000 -0.5 0.5]);
        set(gca,'buttondownfcn','');
        set(tPhys(6),'String','');
        set(tPhys(7),'String',''); 
        key=1;
        while key==1,
         set(gca,'units','normalized');
         [x,y,key]=ginput(1);
          set(tPhys(6),'string',[get(tPhys(6),'string') ' ' num2str(x)]); 
          set(tPhys(7),'string',[get(tPhys(7),'string') ' ' num2str(y)]);  
          sdemo('Draw_lines');
     end
        sdemo('Draw_finish',s);
 

elseif strcmp(action,'Draw_lines')
       tmp=get(gcf,'UserData');
       tPhys=tmp(1:7);
       x=str2num(get(tPhys(6),'String'));
       y=str2num(get(tPhys(7),'String'));
       if length(x)>1,
           for i=1:length(x)-1,
             line([x(i) x(i+1)],[y(i) y(i+1)]);
           end
       end

elseif strcmp(action,'Draw_finish')
       tmp=get(gcf,'UserData');
       tPhys=tmp(1:7);
       N=str2num(get(tPhys(4),'string'));
       x=str2num(get(tPhys(6),'String'));
       y=str2num(get(tPhys(7),'String'));
       sz=length(x);
       x=x-min(x);
       %[x,index]=sort(x);
       %y=y(index);
       
       for i=1:N,      
           tmp=find(x>max(x)/N*i);
           if isempty(tmp), break;end
           j=tmp(1)-1;
           xn(i)=max(x)/N*i;
           yn(i)=y(j)+(y(j+1)-y(j))/(x(j+1)-x(j)+1e-10)*(xn(i)-x(j));
           if i>1,
              if xn(i)<=xn(i-1), xn(i)=xn(i-1);end
	   end   
       end
        if s==2,
          set(tPhys(7),'string',num2str(yn));
          set(tPhys(6),'string',['0' num2str(xn)]); 
        end
          sdemo('solve',1);

end;     %

